# -*- coding: utf-8 -*-
"""
Normalizing of HMHospitales COVID19 database 01
This version contains the second model, used for ICU days prediction
* ONLY FOR PATIENTS WHO HAVE REQUIRED THE ICU * 

@author: varsha
"""
import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt
from sklearn.preprocessing import LabelEncoder
from sklearn.preprocessing import MinMaxScaler
from sklearn.ensemble import RandomForestRegressor
from sklearn import metrics
from sklearn.metrics import recall_score
from sklearn.model_selection import train_test_split, KFold
import numpy as np

df = pd.read_csv('../Database/HM_Hospitales/csv/01.csv')
print(df.describe().T)

print(df.isnull().sum())

#Drop columns with too many null values (~2000 out of 2300)
df_clean = df.drop(["F_ENTRADA_UC/ICU_DATE_IN","F_SALIDA_UCI/ICU_DATE_OUT"], axis=1)

#Drop (not needed) columns with some null values
df_clean = df_clean.drop(["HORA/TIME_CONSTANT_PRIMERA/FIRST_URG/EMERG","HORA/TIME_CONSTANT_ULTIMA/LAST_URG/EMERG",
                          "F_INGRESO/ADMISSION_DATE_URG/EMERG","HORA/TIME_ADMISION/ADMISSION_URG/EMERG", 
                          "F_ALTA/DISCHARGE_DATE_ING", "F_INGRESO/ADMISSION_D_ING/INPAT"], 
                         axis=1)


df_clean['MOTIVO_ALTA/DESTINY_DISCHARGE_ING'].fillna("NO ALTA", inplace=True)

#Drop all rows with at leas one null value (including ICU_DAYS = null = 0)
df_clean = df_clean.dropna()
df_clean = df_clean.drop(df_clean[df_clean["UCI_DIAS/ICU_DAYS"] == 0].index)
print(df_clean.isnull().sum())

## Not dropping any rows by discharge reason, only focusing on ICU days

#Change column names to be readable
df_clean = df_clean.rename(columns={'PATIENT ID':'id',
                         'EDAD/AGE':'age',
                         'SEXO/SEX':'sex',
                         'DIAG ING/INPAT':'covid_diagnosis',
                         'UCI_DIAS/ICU_DAYS':'icu_days',
                         'MOTIVO_ALTA/DESTINY_DISCHARGE_ING':'discharge_reason',
                         'ESPECIALIDAD/DEPARTMENT_URG/EMERG':'department',
                         'DIAG_URG/EMERG':'emergency_diagnosis',
                         'DESTINO/DESTINY_URG/EMERG':'emergency_dept',
                         'TEMP_PRIMERA/FIRST_URG/EMERG':'em_first_temperature',
                         'FC/HR_PRIMERA/FIRST_URG/EMERG':'em_first_hr',
                         'GLU_PRIMERA/FIRST_URG/EMERG':'em_first_glucemia',
                         'SAT_02_PRIMERA/FIRST_URG/EMERG':'em_first_OxSaturation',
                         'TA_MAX_PRIMERA/FIRST/EMERG_URG':'em_first_max_blood_pressure',
                         'TA_MIN_PRIMERA/FIRST_URG/EMERG':'em_first_min_blood_pressure',
                         'FC/HR_ULTIMA/LAST_URG/EMERG':'em_last_hr',
                         'TEMP_ULTIMA/LAST_URG/EMERG':'em_last_temperature',
                         'GLU_ULTIMA/LAST_URG/EMERG':'em_last_glucemia',
                         'SAT_02_ULTIMA/LAST_URG/EMERG':'em_last_OxSaturation',
                         'TA_MAX_ULTIMA/LAST_URGEMERG':'em_last_max_blood_pressure',
                         'TA_MIN_ULTIMA/LAST_URG/EMERG':'em_last_min_blood_pressure'})
print(df_clean.dtypes)

#Change text (object) variables to be numbers
#Randomly done by LabelEncoder (we do this to because text data is more complicated to analyse)

df_clean['sex'] = LabelEncoder().fit_transform(df_clean['sex'])
df_clean['sex'].value_counts()

df_clean['covid_diagnosis'] = LabelEncoder().fit_transform(df_clean['covid_diagnosis'])
df_clean['covid_diagnosis'].value_counts()

df_clean['department'] = LabelEncoder().fit_transform(df_clean['department'])
df_clean['department'].value_counts()

df_clean['emergency_diagnosis']= LabelEncoder().fit_transform(df_clean['emergency_diagnosis'])
df_clean['emergency_diagnosis'].value_counts()

df_clean['emergency_dept'] = LabelEncoder().fit_transform(df_clean['emergency_dept'])
df_clean['emergency_dept'].value_counts()

df_clean['discharge_reason'] = LabelEncoder().fit_transform(df_clean['discharge_reason'])
df_clean['discharge_reason'].value_counts()

print(df_clean.dtypes)
print(df_clean.describe().T)



######################################### REGRESSOR ##############################################
# DEFINING X AND Y

# Y = variables to be predicted (patient progression)
Y = df_clean["icu_days"].values
# X = information we have about the patient
X = df_clean.drop(labels = ["id", "icu_days"], axis=1)
features_list = list(X.columns)

#Scale data to be between 0 and 1
scaler = MinMaxScaler()
scaler.fit(X)
X = scaler.transform(X)

########### INITIAL PREDICTION ###########
#Split dataframe data to get train and test variables (80% train and 20% test)
X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size=0.2, random_state=32)

#Use RANDOM FOREST REGRESSOR to make predictions
model = RandomForestRegressor(n_estimators = 100, random_state = 32)
# Train the model on training data
model.fit(X_train, y_train)
prediction = model.predict(X_test)


#Prediction is done, now let's check the model's accuracy
print("Absolut error = ", metrics.mean_absolute_error(y_test, prediction))
print("Mean squared error = ", metrics.mean_squared_error(y_test, prediction))
print("Root mean squared error = ", np.sqrt(metrics.mean_squared_error(y_test, prediction)))


###################################### END REGRESSOR ###########################################


##################################### CLASS PREDICTION ##########################################

# --------------------------------------- Group of 4 ------------------------------------------
# Show the histogram to check if class classification could be used #
from matplotlib import pyplot as plt
plt.hist(Y, bins=100)

# Divide in 4 groups

X_class_four = X
y_class_four = []

for days in Y:
    if(days < 11):
        y_class_four.append("1-10")
    elif(days < 21):
        y_class_four.append("11-20")
    elif(days < 31):
        y_class_four.append("21-30")
    else:
        y_class_four.append("+30")

y_class_four = np.array(y_class_four)
y_class_four = LabelEncoder().fit_transform(y_class_four)

#Scale data to be between 0 and 1
scaler = MinMaxScaler()
scaler.fit(X_class_four)
X_class_four = scaler.transform(X_class_four)

########### INITIAL PREDICTION ###########
#Split dataframe data to get train and test variables (70% train and 30% test)
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_class_four, y_class_four, test_size=0.3, random_state=42)

#Use RANDOM FOREST CLASSIFIER to make predictions
from sklearn.ensemble import RandomForestClassifier
model = RandomForestClassifier(random_state = 32)
# Train the model on training data
model.fit(X_train_class, y_train_class)
prediction_four = model.predict(X_test_class)

#Prediction is done, now let's check the model's accuracy
print("Accuracy = ", metrics.accuracy_score(y_test_class, prediction_four))

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 4
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_four)
print("Mean IoU for Random Forest = ", IOU_RF.result().numpy())
# 0.17

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[0,3] + values[1,0] + values[2,0] + values[3,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[1,3] + values[0,1] + values[2,1] + values[3,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[2,3] + values[0,2] + values[1,2] + values[3,2])
class4_IoU = values[3,3]/(values[3,3] + values[3,0] + values[3,1] + values[3,2] + values[0,3] + values[1,3] + values[2,3])

print("IoU for class [1-10] is: ", class2_IoU)
print("IoU for class [11-20] is: ", class3_IoU)
print("IoU for class [21-30] is: ", class4_IoU)
print("IoU for class [+30] is: ", class1_IoU)

# --------------------------------------- Group of 4 ------------------------------------------

# --------------------------------------- Group of 3 ------------------------------------------
# Divide in 3 groups

X_class_three = X
y_class_three = []

for days in Y:
    if(days < 11):
        y_class_three.append("1-10")
    elif(days < 26):
        y_class_three.append("11-25")
    else:
        y_class_three.append("+25")

y_class_three = np.array(y_class_three)
y_class_three = LabelEncoder().fit_transform(y_class_three)

y_class_three2 = []

for days in Y:
    if(days < 8):
        y_class_three2.append("1-7") #1
    elif(days < 15):
        y_class_three2.append("8-14") #2
    else:
        y_class_three2.append("+14") #0

y_class_three2 = np.array(y_class_three2)
y_class_three2 = LabelEncoder().fit_transform(y_class_three2)

#Scale data to be between 0 and 1
scaler = MinMaxScaler()
scaler.fit(X_class_three)
X_class_four = scaler.transform(X_class_three)

########### INITIAL PREDICTION ###########
#Split dataframe data to get train and test variables (80% train and 20% test)
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_class_three, y_class_three, test_size=0.3, random_state=42)

#Use RANDOM FOREST CLASSIFIER to make predictions
from sklearn.ensemble import RandomForestClassifier
model = RandomForestClassifier(random_state = 32)
# Train the model on training data
model.fit(X_train_class, y_train_class)
prediction_three = model.predict(X_test_class)

#Prediction is done, now let's check the model's accuracy
print("Accuracy = ", metrics.accuracy_score(y_test_class, prediction_three))
# WENT FROM 0.638 TO 0.66

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 3
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_four)
print("Mean IoU for Random Forest = ", IOU_RF.result().numpy())
# 0.15 to 0.21

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[1,0] + values[2,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[0,1] + values[2,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[0,2] + values[1,2])

print("IoU for class [1-10] is: ", class2_IoU)
print("IoU for class [11-25] is: ", class3_IoU)
print("IoU for class [+25] is: ", class1_IoU)

# --------------------------------------- Group of 3 ------------------------------------------

# --------------------------------------- Group of 2 ------------------------------------------
# Divide in 2 groups

X_class_two = X
y_class_two = []

for days in Y:
    if(days < 16):
        y_class_two.append("1-15") # 1
    else:
        y_class_two.append("+15") # 0

y_class_two = np.array(y_class_two)
y_class_two = LabelEncoder().fit_transform(y_class_two)

y_class_two_7 = []
y_class_two_10 = []

for days in Y:
    if(days < 8):
        y_class_two_7.append("1-7") # 1
    else:
        y_class_two_7.append("+7") # 0
        
for days in Y:
    if(days < 11):
        y_class_two_10.append("1-10") # 1
    else:
        y_class_two_10.append("+10") # 0
        
y_class_two_7 = np.array(y_class_two_7)
y_class_two_7 = LabelEncoder().fit_transform(y_class_two_7)

y_class_two_10 = np.array(y_class_two_10)
y_class_two_10 = LabelEncoder().fit_transform(y_class_two_10)
        
#Scale data to be between 0 and 1
scaler = MinMaxScaler()
scaler.fit(X_class_two)
X_class_two = scaler.transform(X_class_two)

########### INITIAL PREDICTION ###########
#Split dataframe data to get train and test variables (80% train and 20% test)
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_class_two, y_class_two, test_size=0.3, random_state=42)

#Use RANDOM FOREST CLASSIFIER to make predictions
from sklearn.ensemble import RandomForestClassifier
model = RandomForestClassifier(random_state = 32)
# Train the model on training data
model.fit(X_train_class, y_train_class)
prediction_four = model.predict(X_test_class)

#Prediction is done, now let's check the model's accuracy
print("Accuracy = ", metrics.accuracy_score(y_test_class, prediction_four))
# WENT FROM 0.66 TO 0.805

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 2
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_four)
print("Mean IoU for Random Forest = ", IOU_RF.result().numpy())
# 0.21 to 0.508

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[1,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[0,1])

print("IoU for class [1-15] is: ", class2_IoU)
print("IoU for class [+15] is: ", class1_IoU)

# --------------------------------------- Group of 2 ------------------------------------------

##############################################################
        ## METHOD 2: SUPPORT VECTOR CLASSIFICATOR ##
##############################################################

# ----------------------------- Two classes -----------------------------
from sklearn import svm

# Already encoded
X_svm_two = X_class_two
y_svm_two = y_class_two

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_svm_two, y_svm_two, test_size=0.3, random_state=42)
SVM = svm.SVC()
SVM.fit(X_train_class, y_train_class)

#Predict
svm_two_prediction = SVM.predict(X_test_class)

#Accuracy
print("Accuracy SVM 2 = ", metrics.accuracy_score(y_test_class, svm_two_prediction))
#0.83 -> Better than RF, but for each class ?

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 2
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, svm_two_prediction)
print("Mean IoU for SVM = ", IOU_RF.result().numpy())
# 0.41 < RF is better with each class

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[1,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[0,1])

print("IoU for class [1-15] is: ", class2_IoU)
print("IoU for class [+15] is: ", class1_IoU)

# MULTI-CLASS SVM
# ----------------------------- Three classes -----------------------------
from sklearn import svm

# Already encoded
X_svm_three = X_class_three
y_svm_three = y_class_three

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_svm_three, y_svm_three, test_size=0.3, random_state=42)
SVM = svm.LinearSVC()
SVM.fit(X_train_class, y_train_class)

#Predict
svm_three_prediction = SVM.predict(X_test_class)

#Accuracy
print("Accuracy SVM 3 = ", metrics.accuracy_score(y_test_class, svm_three_prediction))
#0.69 -> Better than RF, but for each class ?

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 3
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, svm_three_prediction)
print("Mean IoU for SVM = ", IOU_RF.result().numpy())
# 0.24 > Better than RF for each class too

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[1,0] + values[2,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[0,1] + values[2,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[0,2] + values[1,2])

print("IoU for class [1-10] is: ", class2_IoU)
print("IoU for class [11-25] is: ", class3_IoU)
print("IoU for class [+25] is: ", class1_IoU)

# ----------------------------- Four classes -----------------------------
from sklearn import svm

# Already encoded
X_svm_four = X_class_four
y_svm_four = y_class_four

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_svm_four, y_svm_four, test_size=0.3, random_state=42)
SVM = svm.LinearSVC()
SVM.fit(X_train_class, y_train_class)

#Predict
svm_four_prediction = SVM.predict(X_test_class)

#Accuracy
print("Accuracy SVM 4 = ", metrics.accuracy_score(y_test_class, svm_four_prediction))
#0.75 -> Better than RF, but for each class ?

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 4
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, svm_four_prediction)
print("Mean IoU for SVM = ", IOU_RF.result().numpy())
# 0.19 > Better than RF for each class too (0.15)

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[0,3] + values[1,0] + values[2,0] + values[3,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[1,3] + values[0,1] + values[2,1] + values[3,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[2,3] + values[0,2] + values[1,2] + values[3,2])
class4_IoU = values[3,3]/(values[3,3] + values[3,0] + values[3,1] + values[3,2] + values[0,3] + values[1,3] + values[2,3])

print("IoU for class [1-10] is: ", class2_IoU)
print("IoU for class [11-20] is: ", class3_IoU)
print("IoU for class [21-30] is: ", class4_IoU)
print("IoU for class [+30] is: ", class1_IoU)

##############################################################
                   ## METHOD 3: KNN ##
##############################################################

from sklearn.neighbors import KNeighborsClassifier
knn = KNeighborsClassifier()
# ----------------------------- Four classes -----------------------------
# Already encoded
X_knn_four = X_class_four
y_knn_four = y_class_four

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_svm_four, y_svm_four, test_size=0.3, random_state=42)

#Fitting
knn.fit(X_train_class, y_train_class)
print("Training Accuracy of KNN: ", knn.score(X_train_class,y_train_class))
    
#Predict
prediction_knn_4 = knn.predict(X_test_class)
print("Testing Accuracy KNN 4: ", metrics.accuracy_score(prediction_knn_4,y_test_class))
#Best results so far: 0.75

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 4
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_knn_4)
print("Mean IoU for KNN = ", IOU_RF.result().numpy())
# 0.18 > Better than RF for each class too (0.15)

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[0,3] + values[1,0] + values[2,0] + values[3,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[1,3] + values[0,1] + values[2,1] + values[3,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[2,3] + values[0,2] + values[1,2] + values[3,2])
class4_IoU = values[3,3]/(values[3,3] + values[3,0] + values[3,1] + values[3,2] + values[0,3] + values[1,3] + values[2,3])

print("IoU for class [1-10] is: ", class2_IoU)
print("IoU for class [11-20] is: ", class3_IoU)
print("IoU for class [21-30] is: ", class4_IoU)
print("IoU for class [+30] is: ", class1_IoU)

# ----------------------------- Three classes -----------------------------
# Already encoded
X_knn_three = X_class_three
y_knn_three = y_class_three

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_svm_three, y_svm_three, test_size=0.3, random_state=42)

#Fitting
knn.fit(X_train_class, y_train_class)
print("Training Accuracy of KNN: ", knn.score(X_train_class,y_train_class))
    
#Predict
prediction_knn_3 = knn.predict(X_test_class)
print("Testing Accuracy KNN 3: ", metrics.accuracy_score(prediction_knn_3,y_test_class))
#Same as 4 classes: 0.75

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 3
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_knn_3)
print("Mean IoU for KNN = ", IOU_RF.result().numpy())
# 0.25 > Better than RF && SVM

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[1,0] + values[2,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[0,1] + values[2,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[0,2] + values[1,2])

print("IoU for class [1-10] is: ", class2_IoU)
print("IoU for class [11-25] is: ", class3_IoU)
print("IoU for class [+25] is: ", class1_IoU)
    
# ----------------------------- Two classes -----------------------------
# Already encoded
X_knn_two = X_class_two
y_knn_two = y_class_two

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_knn_two, y_knn_two, test_size=0.3, random_state=42)

#Fitting
knn.fit(X_train_class, y_train_class)
print("Training Accuracy of KNN: ", knn.score(X_train_class,y_train_class))
    
#Predict
prediction_knn_2 = knn.predict(X_test_class)
print("Testing Accuracy KNN 2: ", metrics.accuracy_score(prediction_knn_2,y_test_class))
#BEST OF ALL: 0.82

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 2
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_knn_2)
print("Mean IoU for KNN = ", IOU_RF.result().numpy())
# 0.607 > Better than RF && SVM (BEST)

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[1,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[0,1])

print("IoU for class [1-15] is: ", class2_IoU) #0.88
print("IoU for class [+15] is: ", class1_IoU) #0.33

# BEST RESULTS: KNN WITH TWO CLASSES (overall and by class)
####### KNN WITH TWO CLASSES #######
# Threshold = 7
# Already encoded
X_knn_two_7 = X_class_two
y_knn_two_7 =  y_class_two_7
#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_knn_two_7, y_knn_two_7, test_size=0.3, random_state=42)

#Fitting
knn.fit(X_train_class, y_train_class)
print("Training Accuracy of KNN: ", knn.score(X_train_class,y_train_class))
    
#Predict
prediction_knn_2 = knn.predict(X_test_class)
print("Testing Accuracy KNN 2: ", metrics.accuracy_score(prediction_knn_2,y_test_class))
#0.61 < 15 threshold

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 2
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_knn_2)
print("Mean IoU for KNN = ", IOU_RF.result().numpy())
# 0.41 < worse than 15 threshold

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[1,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[0,1])

print("IoU for class [1-7] is: ", class2_IoU) #0.53
print("IoU for class [+7] is: ", class1_IoU) #0.30

# Threshold = 10
# Already encoded
X_knn_two_10 = X_class_two
y_knn_two_10 =  y_class_two_10
#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_knn_two_10, y_knn_two_10, test_size=0.3, random_state=42)

#Fitting
knn.fit(X_train_class, y_train_class)
print("Training Accuracy of KNN: ", knn.score(X_train_class,y_train_class))
    
#Predict
prediction_knn_2 = knn.predict(X_test_class)
print("Testing Accuracy KNN 2: ", metrics.accuracy_score(prediction_knn_2,y_test_class))
#0.77 < 15 threshold

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 2
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_knn_2)
print("Mean IoU for KNN = ", IOU_RF.result().numpy())
# 0.51 < worse than 15 threshold

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[1,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[0,1])

print("IoU for class [1-10] is: ", class2_IoU) #0.75
print("IoU for class [+10] is: ", class1_IoU) #0.27
    

## SEPARATION IN THREE GROUPS KNN ##
# Already encoded
X_knn_three2 = X_class_three
y_knn_three2 = y_class_three2

#Let's mantain 70% - 30%
X_train_class, X_test_class, y_train_class, y_test_class = train_test_split(X_knn_three2, y_knn_three2, test_size=0.3, random_state=42)

#Fitting
knn.fit(X_train_class, y_train_class)
print("Training Accuracy of KNN: ", knn.score(X_train_class,y_train_class))
    
#Predict
prediction_knn_3 = knn.predict(X_test_class)
print("Testing Accuracy KNN 3: ", metrics.accuracy_score(prediction_knn_3,y_test_class))
#0.67

# Let's check IoU
from keras.metrics import MeanIoU
num_classes = 3
IOU_RF = MeanIoU(num_classes=num_classes)  
IOU_RF.update_state(y_test_class, prediction_knn_3)
print("Mean IoU for KNN = ", IOU_RF.result().numpy())
# 0.30 < than two classes

values = np.array(IOU_RF.get_weights()).reshape(num_classes, num_classes)
print(values)

class1_IoU = values[0,0]/(values[0,0] + values[0,1] + values[0,2] + values[1,0] + values[2,0])
class2_IoU = values[1,1]/(values[1,1] + values[1,0] + values[1,2] + values[0,1] + values[2,1])
class3_IoU = values[2,2]/(values[2,2] + values[2,0] + values[2,1] + values[0,2] + values[1,2])

print("IoU for class [1-7] is: ", class2_IoU)
print("IoU for class [8-14] is: ", class3_IoU)
print("IoU for class [+14] is: ", class1_IoU)    
    
    
    
    
    